@@ -529,25 +529,27 @@ M3Result SuppressLookupFailure(M3Result i_result)
         return i_result;
 }
 
+m3_wasi_context_t* m3_GetModuleWasiContext(IM3Module module) {
+    return module->wasi;
+}
+
 m3_wasi_context_t* m3_GetWasiContext()
 {
     return wasi_context;
 }
 
+void m3_FreeWasi(m3_wasi_context_t* wasi)
+{
+    m3_Free(wasi);
+}
 
-M3Result  m3_LinkEspWASI  (IM3Module module)
+static inline
+M3Result  _linkWASI  (IM3Module module, wasi_context_t* wasi_context)
 {
     M3Result result = m3Err_none;
 
     // TODO: Preopen dirs
 
-    if (!wasi_context) {
-        wasi_context = (m3_wasi_context_t*)malloc(sizeof(m3_wasi_context_t));
-        wasi_context->exit_code = 0;
-        wasi_context->argc = 0;
-        wasi_context->argv = 0;
-    }
-
     static const char* namespaces[2] = { "wasi_unstable", "wasi_snapshot_preview1" };
 
     // fd_seek is incompatible
@@ -612,5 +614,26 @@ _catch:
     return result;
 }
 
+M3Result m3_LinkModuleEspWASI (IM3Module module)
+{
+    if (NULL == module->wasi) {
+        module->wasi = m3_AllocStruct(m3_wasi_context_t);
+    }
+    else {
+        return "WASI already linked";
+    }
+
+    return _linkWASI(module, module->wasi);
+}
+
+M3Result  m3_LinkEspWASI  (IM3Module module)
+{
+    if (!wasi_context) {
+        wasi_context = m3_AllocStruct(m3_wasi_context_t);
+    }
+
+    return _linkWASI(module, wasi_context);
+}
+
 #endif // ESP32
 
